/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008-2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef LISTBOX_ITEM_H
#define LISTBOX_ITEM_H

#include "base/util/StringBuffer.h"
#include "base/util/ArrayElement.h"
#include "commonDefs.h"

using namespace Funambol;


/**
 * Represents an item in the UI listbox (one row in the main screen list).
 * The FunambolAppView has an arraylist of these objects, one for each
 * source visible.
 */
class ListboxItem : public ArrayElement
{

private:
    
    TInt          sourceId;         /**< The corresponding source ID        */
    StringBuffer  label;            /**< The label displayed (first line)   */
    StringBuffer  status;           /**< The status displayed (second line) */
    bool          enabled;          /**< true = enabled, false = greyed     */
    int           iconIndex;        /**< The normal icon index in the icons array  */
    int           iconDimmedIndex;  /**< The dimmed (greyed) icon index in the icons array  */
    
    
public:
    
    /// Constructor: must pass ALL the params to init the item
    ListboxItem(TInt aSourceId, const StringBuffer& aLabel, const StringBuffer& aStatus, 
                bool aEnabled, TInt aIconIndex, TInt aIconDimmedIndex) :
                sourceId(aSourceId), label(aLabel), status(aStatus), 
                enabled(aEnabled), iconIndex(aIconIndex), 
                iconDimmedIndex(aIconDimmedIndex) {}

    ~ListboxItem() {}
    
    // get methods
    TInt         getSourceId()          { return sourceId;        }
    StringBuffer getLabel()             { return label;           }
    StringBuffer getStatus()            { return status;          }
    bool         getEnabled()           { return enabled;         }
    int          getIconIndex()         { return iconIndex;       }
    int          getIconDimmedIndex()   { return iconDimmedIndex; }
    
    // set methods - other params are constant and set only from constructor
    void setStatus  (const StringBuffer& aStatus) { status = aStatus;   }
    void setEnabled (bool aEnabled)               { enabled = aEnabled; }
    
    // from ArrayElement
    ArrayElement* clone() {
        return new ListboxItem(sourceId, label, status, enabled, iconIndex, iconDimmedIndex);
    }
};

#endif
