/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include <e32std.h>

#include "base/SymbianLog.h"
#include "FileObserver.h"
#include "ClientPush.h"
#include "commonDefs.h"

using namespace Funambol;


CFileObserver::CFileObserver(CClientPush* parent, const TInt aID) 
                                                  : CActive(CActive::EPriorityStandard), 
                                                    iClientPush(parent),
                                                    ID(aID),
                                                    iListening(false) {

    // Connect the FsSession and add this AO to the ActiveScheduler
    LOG.debug("CP: Initializing FileObserver (ID = %d)...", ID);
    iFsSession.Connect();
    CActiveScheduler::Add(this);
}

CFileObserver::~CFileObserver() 
{
    CActive::Cancel();
    iFsSession.Close();
}


void CFileObserver::start()
{
    if (iPath.empty()) {
        LOG.error("CP: cannot start FileObserver: empty path");
        return;
    }
	
    LOG.info("CP: start watching for file modifications on '%s'", iPath.c_str());
    
    RBuf folderPath;
    folderPath.Assign(stringBufferToNewBuf(iPath));
    
    Cancel();   // to cancel any outstanding request
    iFsSession.NotifyChange(ENotifyFile, iStatus, folderPath);
    SetActive();
    
    folderPath.Close();
    iListening = true;
}


void CFileObserver::stop()
{
    LOG.info("CP: stop watching for file modifications on '%s'", iPath.c_str());
    iFsSession.NotifyChangeCancel();
    iListening = false;
}


void CFileObserver::setPath(const StringBuffer& aPath)
{
    //LOG.debug("%s", __PRETTY_FUNCTION__);
    if (aPath.empty()) {
        LOG.error("CP: empty fileObserver path");
        return;
    }
    
    if (isRunning() && (aPath != iPath)) {
        // NotifyChange() is running on a different path: stop and restart it!
        stop();
        iPath = aPath;
        start();
    }
    else {
        iPath = aPath; 
    }
}


void CFileObserver::RunL()
{
    //LOG.debug("RunL - status = %d", iStatus.Int());
    if(iStatus == KErrNone) 
    {
        // CALLBACK!
        LOG.debug("CP: detected file change in '%s'!", iPath.c_str());
        iClientPush->OnFileNotification(ID);
        
        start();  // restart observer for new notifications
    }
    else {
        LOG.error("CFileObserver::RunL() error: status = %d", iStatus.Int());
    }
}

void CFileObserver::DoCancel()
{
    iFsSession.NotifyChangeCancel(); 
}

TInt CFileObserver::RunError(TInt aError) {
    LOG.error("CFileObserver RunL() error: %d", aError);
    return 0;
}

